/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 4; tab-width: 4 -*- */
/* vim:set et ai sw=4 ts=4 sts=4: tw=80 cino="(0,W2s,i2s,t0,l1,:0" */

import QtQuick 1.0
import "components"

Overlay {
    property alias launchPhone: extraPhone.visible

    signal skipTutorial();
    signal phoneClicked();

    RefButton {
        anchors.top: parent.top;
        anchors.topMargin: swipeWidth;
        anchors.right: parent.right;
        anchors.rightMargin: 26; // Layout spec
        text: qtn_tuto_skip;

        onTapped: {
            skipTutorial();
            state = "hidden";
        }
    }
    
    // http://bugreports.qt.nokia.com/browse/QTBUG-18901
    // Thus positioned by hard coded values...
    MouseArea {
        id: extraPhone;
        visible: false;
        x: 12;
        y: 60;
        height: 148; // Home Layout spec
        width: 114; // Home Layout spec + 4 pixel cheat because could not found padding property
        /* cheat to have the 4 pixel padding specified in Home Layout spec */
        anchors.leftMargin: 2;
        anchors.rightMargin: 2;

        onPressed: {
            extraPhoneDim.opacity = 1-homeDim;
        }
        onExited: {
            extraPhoneDim.opacity = 0;
        }
        onClicked: {
            phoneClicked();
            state = "hidden";
            extraPhoneDim.opacity = 0;
        }

        Image {
            id: extraPhoneIcon;
            width: 80; // Home Layout spec
            height: 80; // Home Layout spec
            anchors.horizontalCenter: parent.horizontalCenter;
            source: appsIconRoot + "icon-l-telephony.png";
        }

        Text {
            id: extraPhoneText;
            text: phoneUiNameText;
            anchors.topMargin: 10;
            anchors.top: extraPhoneIcon.bottom;
            width: parent.width;
            horizontalAlignment: Text.AlignHCenter;
            color: "white";
            font.family: defaultFont;
            font.pixelSize: 16;
            font.weight: Font.Normal;
        }

        Rectangle {
            id: extraPhoneDim;
            anchors.fill: parent;
            color: "black";
            opacity: 0;
        }
    }

    Text {
        width: parent.width;
        anchors.topMargin: 40; // Layout spec
        anchors.left: parent.left;
        anchors.leftMargin: 40; // Layout spec
        anchors.right: parent.right;
        anchors.rightMargin: 40; // Layout spec
        anchors.bottom: bottomText.top;
        anchors.bottomMargin: 40; // Layout spec

        text: launchPhone ? qtn_tuto_launcher_text_1 : "";
        wrapMode: Text.WordWrap;
        horizontalAlignment: Text.AlignHCenter;
        font.family: instrFont;
        font.pixelSize: instrFontSize;
        font.weight: Font.Normal;
        color: "white";
    }
    
    Text {
        id: bottomText;

        width: parent.width;
        anchors.topMargin: 0; // Layout spec
        anchors.left: parent.left;
        anchors.leftMargin: 40; // Layout spec
        anchors.right: parent.right;
        anchors.rightMargin: 40; // Layout spec
        anchors.bottom: parent.bottom;
        anchors.bottomMargin: 60; // Layout spec
        
        text: launchPhone ? qtn_tuto_launcher_text_2 : qtn_tuto_launcher_text_3;
        wrapMode: Text.WordWrap;
        horizontalAlignment: Text.AlignHCenter;
        font.family: instrFont;
        font.pixelSize: instrBigFontSize;
        font.weight: Font.Light;
        color: "white";
    }
}



