/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 4; tab-width: 4 -*- */
/* vim:set et ai sw=4 ts=4 sts=4: tw=80 cino="(0,W2s,i2s,t0,l1,:0" */

import QtQuick 1.0
import "components"

Rectangle {
    id: container
    objectName: "MainElement"
    
    width: 480
    height: 854
    state: "welcomeState"
    color: "black"

    /* Signaling for C++ application. */
    signal endViewIn()
    signal endViewOut()

    /*
     * Global consts
     */
    property string defaultFont: "Nokia Pure Text"
    property string buttonFont: "Nokia Pure Text"
    property int buttonFontSize: 24; // guess
    property string instrFont: "Nokia Pure Text"
    property int instrFontSize: 24; // Layout spec
    property int instrBigFontSize: 40; // Layout spec

    property int fingerXForPanHelp: 280;
    property int fingerYForPanHelp: 260;
    
    property string blueColor: "#1080DD";

    property string appsIconRoot: "/usr/share/themes/blanco/meegotouch/icons/"
    
    // Image to be used as screen lock background set by binary based on gconf values
    property string screenLockImageFile: ""

    /* Overriding text align mode because for arabic locale we
     * have special layout alignments:
     * Align every non fielded text to the left. */
    property bool layoutLeftToRightMode: true;


    /*
     * Pan parameters
     */
    property int panDuration: 400
    property int panThreshold: 25
    property real cancelShortEdgePan: 0.27

    /*
     * Swipe parameters, replaced by values from mcompositor-swipe.conf in C++ code
     */
    property real homeZoom: 0.8
    property real homeDim: 0.3
    property int swipeDuration: 250 // Motion Guide 2.1
    property int swipeWidth: 12
    property int swipeThreshold: 10
    property real cancelShortEdgeSwipe: 0.5
    property real cancelLongEdgeSwipe: 0.25

    /*
     * Localization
     */

    /* For multiple screens */
    property string qtn_tuto_skip: qsTr("qtn_tuto_skip");
    /* Welcome screen */
    property string qtn_tuto_initial_text_1: qsTr("qtn_tuto_initial_text_1");
    property string qtn_tuto_start_tour: qsTr("qtn_tuto_start_tour");
    /* Lock screen */
    property string qtn_tuto_initial_text_2: qsTr("qtn_tuto_initial_text_2");
    property string currentDate: "date";
    property string currentTime: "time";
    /* Launcher screen 1 */
    property string qtn_tuto_launcher_text_1: qsTr("qtn_tuto_launcher_text_1")
    property string qtn_tuto_launcher_text_2: qsTr("qtn_tuto_launcher_text_2")
    property string phoneUiNameText: "";
    /* Phone screen */
    property string qtn_tuto_phone_text_1: qsTr("qtn_tuto_phone_text_1")
    property string qtn_tuto_phone_text_2: qsTr("qtn_tuto_phone_text_2")
    /* Phone Closing screen */
    property string qtn_tuto_tap_phone_again: qsTr("qtn_tuto_tap_phone_again")
    property string qtn_tuto_phone_text_3: qsTr("qtn_tuto_phone_text_3")
    /* Launcher screen 2 */
    property string qtn_tuto_launcher_text_3: qsTr("qtn_tuto_launcher_text_3")
    /* Recents screen */
    property string qtn_tuto_recents_text: qsTr("qtn_tuto_recents_text")
    property string qtn_tuto_pan_to_contintue: qsTr("qtn_tuto_pan_to_contintue")
    /* Events screen */
    property string currentDay: "";
    property string currentDateWithoutDay: "";
    property string qtn_tuto_events_text: qsTr("qtn_tuto_events_text")
    property string qtn_tuto_end: qsTr("qtn_tuto_end")
    property string qtn_tuto_notifications_title: qsTr("qtn_tuto_notifications_title")
    property string qtn_tuto_clear: qsTr("qtn_tuto_clear")
    property string qtn_tuto_news_feed: qsTr("qtn_tuto_news_feed")
    property string qtn_tuto_refresh: qsTr("qtn_tuto_refresh")
    property string eventsViewEventMinuteAgoText: qsTr("qtn_comm_time_minute_ago")
    property string qtn_tuto_fake_name_1: qsTr("qtn_tuto_fake_name_1")
    property string qtn_tuto_missed_call: qsTr("qtn_tuto_missed_call")
    property string qtn_tuto_fake_name_2: qsTr("qtn_tuto_fake_name_2")
    property string qtn_tuto_voicemail: qsTr("qtn_tuto_voicemail")
    property string qtn_tuto_fake_name_3: qsTr("qtn_tuto_fake_name_3")
    property string qtn_tuto_fake_message_text: qsTr("qtn_tuto_fake_message_text")
    property string qtn_tuto_fake_name_4: qsTr("qtn_tuto_fake_name_4")
    property string qtn_tuto_fake_feeds_text: qsTr("qtn_tuto_fake_feeds_text")
    /* End screen */
    property string qtn_tuto_end_text_title: qsTr("qtn_tuto_end_text_title");
    property string qtn_tuto_end_text: qsTr("qtn_tuto_end_text");
    property string qtn_tuto_end_text_2: qsTr("qtn_tuto_end_text_2");
    property string qtn_tuto_start_again: qsTr("qtn_tuto_start_again");

    /* Convention for Z ordering */
    property int zOrderDesktop: 0; // appsgrid, recents, events
    property int zOrderApplication: 1; // phone view
    property int zOrderEnd: 2; // ending lock view
    property int zOrderLock: 3; // starting lock view
    property int zOrderOverlay: 4; // overlays
    property int zOrderUser: 5; // fingerprint

    Rectangle {
        id: fingerPrint;
        width: 80;
        height: 80;
        radius: 40;
        color: blueColor;
        visible: false;
        z: zOrderUser;
    }

    WelcomeView {
        id: welcomeView;
        height: parent.height;
        width: parent.width;
        x: 0;
        y: 0;
        z: zOrderApplication;
        visible: true;

        onStartTutorial: {
            //console.log("welcomeView.onEndTutorial");
            container.state = "lockState";
            openAppFromWelcomeAnim.start();
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromWelcomeAnim
            ParallelAnimation {
                PropertyAnimation { target: welcomeView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: welcomeView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                welcomeView.visible = false;
            } }
        }
    }

    LockScreen {
        id: lockView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderLock;
        visible: false;
        
        onSwipeBegin: {
            //console.log("lockView.onSwipeBegin");
            abortSwipeToLaunchPhoneAnim.stop()
            launchPhoneView.visible = true;
            launchPhoneView.x = 0;
            launchPhoneView.y = 0;
            launchPhoneView.scale = homeZoom;
            launchPhoneView.opacity = homeDim;
            lockView.stopView();
            lockOverlay.stopOverlay();
        }
        onSwipeProgress: {
            launchPhoneView.scale = homeZoom + (1-homeZoom) * progress;
            launchPhoneView.opacity = homeDim + (1-homeDim) * progress;
            /*console.log("lockView.onSwipeProgress: " + progress +
                        "; launchPhoneView: .scale: " + launchPhoneView.scale +
                        "; .opacity: "                + launchPhoneView.opacity +
                        "; .visible: "                + launchPhoneView.visible +
                        "; .x: "                      + launchPhoneView.x +
                        "; .y: "                      + launchPhoneView.y);*/
        }
        onSwipeEnd: {
            //console.log("lockView.onSwipeEnd");
            container.state = "launchPhoneState";
        }
        onSwipeAbort: {
            //console.log("lockView.onSwipeAbort");
            abortSwipeToLaunchPhoneAnim.start();
            lockOverlay.state = "visible";
        }

        onStartHelp: {
            lockView.fingerPrintParent = lockOverlay;
        }

        SequentialAnimation {
            id: abortSwipeToLaunchPhoneAnim
            ParallelAnimation {
                PropertyAnimation { target: launchPhoneView; property: "scale"; to: homeZoom; easing.type: Easing.OutQuint; duration: swipeDuration;}
                NumberAnimation { target: launchPhoneView; property: "opacity"; to: homeDim; easing.type: Easing.OutQuint; duration: swipeDuration;}
            }
            ScriptAction { script: launchPhoneView.visible = false; }
        }
    }

    LockOverlay {
        id: lockOverlay;
    }

    AppsGrid {
        id: launchPhoneView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderDesktop
        visible: false
        scale: homeZoom
        opacity: homeDim
        lockPan: true;
    }

    AppsGridOverlay {
        id: launchPhoneOverlay;
        launchPhone: true;
        dimmer: "images/meegotouch-tutorial-dimming-launcher.png";

        onSkipTutorial: {
            //console.log("launchPhoneOverlay.onSkipTutorial");
            launchPhoneView.enabled = false;
            stopOverlay();
            openAppFromLaunchPhoneViewAnim.stop();
            container.state = "endState";
            openAppFromLaunchPhoneViewAnim.start();
            container.endViewIn()
        }
        onPhoneClicked: {
            //console.log("launcherView.onPhoneClicked");
            launchPhoneView.enabled = false;
            stopOverlay();
            openAppFromLaunchPhoneViewAnim.stop();
            container.state = "phoneState";
            openAppFromLaunchPhoneViewAnim.start();
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromLaunchPhoneViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: launchPhoneView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: launchPhoneView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                launchPhoneView.visible = false;
            } }
        }
    }

    Phone {
        id: phoneView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderApplication
        visible: false

        onSwipeBegin: {
            //console.log("phoneView.onSwipeBegin");
            abortSwipeToLauncherAnim.stop();
            launcherView.visible = true;
            launcherView.x = 0;
            launcherView.y = 0;
            launcherView.scale = homeZoom;
            launcherView.opacity = homeDim;
            phoneView.stopView();
            phoneOverlay.stopOverlay();
        }
        onSwipeProgress: {
            launcherView.scale = homeZoom + (1-homeZoom) * progress;
            launcherView.opacity = homeDim + (1-homeDim) * progress;
            /*console.log("phoneView.onSwipeProgress: " + progress +
                        "; launcherView: .scale: "    + launcherView.scale +
                        "; .opacity: "                + launcherView.opacity +
                        "; .visible: "                + launcherView.visible +
                        "; .x: "                      + launcherView.x +
                        "; .y: "                      + launcherView.y);*/
        }
        onSwipeEnd: {
            //console.log("phoneView.onSwipeEnd");
            container.state = "launcherState";
        }
        onSwipeAbort: {
            //console.log("phoneView.onSwipeAbort");
            abortSwipeToLauncherAnim.start();
            phoneOverlay.state = "visible";
        }

        onStartHelp: {
            phoneView.fingerPrintParent = phoneOverlay;
        }

        SequentialAnimation {
            id: abortSwipeToLauncherAnim
            ParallelAnimation {
                PropertyAnimation {target: launcherView; property: "scale"; to: homeZoom; easing.type: Easing.OutQuint; duration: swipeDuration;}
                NumberAnimation { target: launcherView; property: "opacity"; to: homeDim; easing.type: Easing.OutQuint; duration: swipeDuration;}
            }
            ScriptAction { script: launcherView.visible = false; }
        }
    }

    PhoneOverlay {
        id: phoneOverlay;
        
        onSkipTutorial: {
            //console.log("phoneOverlay.onSkipTutorial");
            phoneView.stopView();
            phoneView.enabled = false;
            stopOverlay();
            openAppFromPhoneViewAnim.stop();
            container.state = "endState";
            openAppFromPhoneViewAnim.start();
            container.endViewIn()
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromPhoneViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: phoneView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: phoneView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                phoneView.visible = false;
            } }
        }
    }

    AppsGrid {
        id: launcherView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderDesktop
        scale: homeZoom
        opacity: homeDim
        visible: false
        enabled: false;

        onPanBegin: {
            //console.log("launcherView.onPanBegin");
//            launcherHelpAnimAssistant.stop();
            abortPanToRecentsAnim.stop();
            recentsOpenPhoneView.x = width;
            recentsOpenPhoneView.y = 0;
            recentsOpenPhoneView.scale = 1;
            recentsOpenPhoneView.opacity = 1;
            recentsOpenPhoneView.visible = true;
            launcherOverlay.stopOverlay();
            launcherHelpAnim.stop();
            launcherHelpAnimTimer.stop();
            launcherOverlay.stopOverlay();
            launcherView.x = 0;
            launcherView.y = 0;
            fingerPrint.visible = false;
        }
        onPanProgress: {
            recentsOpenPhoneView.x += dx;
            /*console.log("launcherView.onPanProgress: " + dx +
                        "; recentsOpenPhoneView: .scale: " + recentsOpenPhoneView.scale +
                        "; .opacity: "                 + recentsOpenPhoneView.opacity +
                        "; .visible: "                 + recentsOpenPhoneView.visible +
                        "; .x: "                       + recentsOpenPhoneView.x +
                        "; .y: "                       + recentsOpenPhoneView.y);*/
        }
        onPanEnd: {
            //console.log("launcherView.onPanEnd");
            container.state = "recentsOpenPhoneState";
        }
        onPanAbort: {
            //console.log("launcherView.onPanAbort");
            abortPanToRecentsAnim.start();
            launcherOverlay.state = "visible";
            launcherHelpAnimTimer.start();
        }
/*
        SequentialAnimation {
            id: launcherHelpAnimAssistant;
            ScriptAction { script: {
                recentsOpenPhoneView.x = width;
                recentsOpenPhoneView.y = 0;
                recentsOpenPhoneView.scale = 1;
                recentsOpenPhoneView.opacity = 1;
                recentsOpenPhoneView.visible = true;
            } }
            PropertyAnimation { target: recentsOpenPhoneView; property: "x"; to: parent.width/2; easing.type: Easing.InOutQuint; duration: 1200 }
            PropertyAnimation { target: recentsOpenPhoneView; property: "x"; to: width; easing.type: Easing.InOutQuint; duration: 600 }
        }
*/
        SequentialAnimation {
            id: abortPanToRecentsAnim
            PropertyAnimation {target: recentsOpenPhoneView; property: "x"; to: width; easing.type: Easing.OutQuint; duration: panDuration;}
            ScriptAction { script: {
                recentsOpenPhoneView.visible = false;
                //console.log("abortPanToRecentsAnim: hide recentsOpenPhone");
            } }
        }

        Timer {
            id: launcherHelpAnimTimer;
            interval: 3000;
            onTriggered: launcherHelpAnim.start();
        }

        SequentialAnimation {
            id: launcherHelpAnim;
            ScriptAction { script: {
                recentsOpenPhoneView.x = width;
                recentsOpenPhoneView.y = 0;
                recentsOpenPhoneView.scale = 1;
                recentsOpenPhoneView.opacity = 1;
                recentsOpenPhoneView.visible = true;
                fingerPrint.x = fingerXForPanHelp;
                fingerPrint.y = fingerYForPanHelp;
                fingerPrint.opacity = 0;
                fingerPrint.visible = true;
            } }

            PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0.5; easing.type: Easing.OutExpo; duration: 250 }
            
            ParallelAnimation {
                PropertyAnimation { target: launcherView; property: "x"; to: -width/2; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: recentsOpenPhoneView; property: "x"; to: width/2; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: fingerPrint; property: "x"; to: fingerXForPanHelp-(width/2); easing.type: Easing.InOutQuint; duration: 1200 }
            }
            PropertyAnimation { target: launcherView; property: "x"; to: -width/2; duration: 600 } // fake anim to wait
            ParallelAnimation {
                PropertyAnimation { target: launcherView; property: "x"; to: 0; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: recentsOpenPhoneView; property: "x"; to: width; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: fingerPrint; property: "x"; to: fingerXForPanHelp; easing.type: Easing.InOutQuint; duration: 1200 }
            }
            
            PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 250 }

            ScriptAction { script: {
                launcherHelpAnimTimer.start();
                fingerPrint.visible = false;
            } }
        }
    }

    AppsGridOverlay {
        id: launcherOverlay;
        
        onSkipTutorial: {
            //console.log("launcherView.onSkipTutorial");
            launcherHelpAnim.stop();
            launcherHelpAnimTimer.stop();
            launcherOverlay.stopOverlay();
            launcherView.x = 0;
            launcherView.y = 0;
            launcherView.enabled = false;
            recentsOpenPhoneView.x = width;
            recentsOpenPhoneView.y = 0;
            recentsOpenPhoneView.visible = false;
            fingerPrint.visible = false;
            stopOverlay();
            openAppFromLauncherViewAnim.stop();
            container.state = "endState";
            openAppFromLauncherViewAnim.start();
            container.endViewIn()
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromLauncherViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: launcherView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: launcherView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                launcherView.visible = false;
            } }
        }
    }

    RecentsOpenPhone {
        id: recentsOpenPhoneView
        height: parent.height
        width: parent.width
        x: parent.width
        y: 0
        z: zOrderDesktop
        visible: false
        enabled: false
        scale: homeZoom
        opacity: homeDim
        lockPan: true;
    }

    RecentsOpenPhoneOverlay {
        id: recentsOpenPhoneOverlay;
        dimmer: "images/meegotouch-tutorial-dimming-launcher.png";

        onSkipTutorial: {
            //console.log("recentsOpenPhoneOverlay.onSkipTutorial");
            recentsOpenPhoneOverlay.stopOverlay();
            recentsOpenPhoneView.x = 0;
            recentsOpenPhoneView.y = 0;
            recentsOpenPhoneView.enabled = false;
            eventsView.x = width;
            eventsView.y = 0;
            eventsView.visible = false;
            fingerPrint.visible = false;
            openPhoneFromRecentsViewAnim.stop();
            container.state = "endState";
            openPhoneFromRecentsViewAnim.start();
            container.endViewIn()
        }
        onOpenPhoneClicked: {
            //console.log("recentsOpenPhoneOverlay.onPhoneClicked");
            
            //recentsOpenPhoneOverlay.stopOverlay();
            //recentsOpenPhoneView.x = 0;
            //recentsOpenPhoneView.y = 0;

            recentsOpenPhoneView.enabled = false;
            stopOverlay();
            openPhoneFromRecentsViewAnim.stop();
            container.state = "phoneClosingState";
            openPhoneFromRecentsViewAnim.start();
        }
        
        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openPhoneFromRecentsViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: recentsOpenPhoneView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: recentsOpenPhoneView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                recentsOpenPhoneView.visible = false;
            } }
        }
    }

    PhoneClosing {
        id: phoneClosingView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderApplication
        visible: false

        onSwipeBegin: {
            //console.log("phoneClosingView.onSwipeBegin");
            abortSwipeToRecentsAnim.stop();            
            recentsView.visible = true;
            recentsView.x = 0;
            recentsView.y = 0;
            recentsView.scale = homeZoom;
            recentsView.opacity = homeDim;
            recentsView.reset();
            phoneClosingView.stopView();
            phoneClosingOverlay.stopOverlay();
        }
        onSwipeProgress: {
            recentsView.scale = homeZoom + (1-homeZoom) * progress;
            recentsView.opacity = homeDim + (1-homeDim) * progress;
            /*console.log("phoneClosingView.onSwipeProgress: " + progress +
                        "; launcherView: .scale: "    + launcherView.scale +
                        "; .opacity: "                + launcherView.opacity +
                        "; .visible: "                + launcherView.visible +
                        "; .x: "                      + launcherView.x +
                        "; .y: "                      + launcherView.y);*/
        }
        onSwipeEnd: {
            //console.log("phoneClosingView.onSwipeEnd");
            container.state = "recentsState";
        }
        onSwipeAbort: {
            //console.log("phoneClosingView.onSwipeAbort");
            abortSwipeToRecentsAnim.start();
            phoneClosingOverlay.state = "visible";
        }

        onStartHelp: {
            phoneClosingView.fingerPrintParent = phoneClosingOverlay;
        }

        SequentialAnimation {
            id: abortSwipeToRecentsAnim
            ParallelAnimation {
                PropertyAnimation {target: recentsView; property: "scale"; to: homeZoom; easing.type: Easing.OutQuint; duration: swipeDuration;}
                NumberAnimation { target: recentsView; property: "opacity"; to: homeDim; easing.type: Easing.OutQuint; duration: swipeDuration;}
            }
            ScriptAction { script: recentsView.visible = false; }
        }
        
    }
    
    PhoneClosingOverlay {
        id: phoneClosingOverlay;
        
        onSkipTutorial: {
            //console.log("phoneClosingOverlay.onSkipTutorial");
            phoneClosingView.stopView();
            phoneClosingView.enabled = false;
            stopOverlay();
            openAppFromPhoneClosingViewAnim.stop();
            container.state = "endState";
            openAppFromPhoneClosingViewAnim.start();
            container.endViewIn()
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromPhoneClosingViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: phoneClosingView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: phoneClosingView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                phoneClosingView.visible = false;
            } }
        }
    }

    Recents {
        id: recentsView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderDesktop
        scale: homeZoom
        opacity: homeDim
        visible: false
        enabled: false;

        onPanBegin: {
            //console.log("recentsView.onPanBegin");
            abortPanToEventsAnim.stop();
            eventsView.x = width;
            eventsView.y = 0;
            eventsView.scale = 1;
            eventsView.opacity = 1;
            eventsView.visible = true;
            recentsOverlay.stopOverlay();
            recentsHelpAnim.stop();
            recentsHelpAnimTimer.stop();
            recentsView.x = 0;
            recentsView.y = 0;
            fingerPrint.visible = false;
        }
        onPanProgress: {
            eventsView.x += dx;
            /*console.log("recentsView.onPanProgress: " + dx +
                        "; eventsView: .scale: "      + eventsView.scale +
                        "; .opacity: "                + eventsView.opacity +
                        "; .visible: "                + eventsView.visible +
                        "; .x: "                      + eventsView.x +
                        "; .y: "                      + eventsView.y);*/
        }
        onPanEnd: {
            //console.log("recentsView.onPanEnd");
            container.state = "eventsState";
        }
        onPanAbort: {
            //console.log("recentsView.onPanAbort");
            abortPanToEventsAnim.start();
            recentsOverlay.state = "visible";
            recentsHelpAnimTimer.start();
        }

        SequentialAnimation {
            id: abortPanToEventsAnim
            PropertyAnimation {target: eventsView; property: "x"; to: width; easing.type: Easing.OutQuint; duration: panDuration;}
            ScriptAction { script: eventsView.visible = false; }
        }

        
        Timer {
            id: recentsHelpAnimTimer;
            interval: 4000;
            onTriggered: recentsHelpAnim.start();
        }

        SequentialAnimation {
            id: recentsHelpAnim;
            ScriptAction { script: {
                eventsView.x = width;
                eventsView.y = 0;
                eventsView.scale = 1;
                eventsView.opacity = 1;
                eventsView.visible = true;
                fingerPrint.x = fingerXForPanHelp;
                fingerPrint.y = fingerYForPanHelp;
                fingerPrint.opacity = 0;
                fingerPrint.visible = true;
            } }

            PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0.5; easing.type: Easing.OutExpo; duration: 250 }
            
            ParallelAnimation {
                PropertyAnimation { target: recentsView; property: "x"; to: -width/2; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: eventsView; property: "x"; to: width/2; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: fingerPrint; property: "x"; to: fingerXForPanHelp-(width/2); easing.type: Easing.InOutQuint; duration: 1200 }
            }
            PropertyAnimation { target: recentsView; property: "x"; to: -width/2; duration: 600 } // fake anim to wait
            ParallelAnimation {
                PropertyAnimation { target: recentsView; property: "x"; to: 0; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: eventsView; property: "x"; to: width; easing.type: Easing.InOutQuint; duration: 1200 }
                PropertyAnimation { target: fingerPrint; property: "x"; to: fingerXForPanHelp; easing.type: Easing.InOutQuint; duration: 1200 }
            }
            
            PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 250 }

            ScriptAction { script: {
                recentsHelpAnimTimer.start();
                fingerPrint.visible = false;
            } }
        }
    }

    RecentsOverlay {
        id: recentsOverlay;
        
        onSkipTutorial: {
            //console.log("recentsOverlay.onSkipTutorial");
            recentsHelpAnim.stop();
            recentsHelpAnimTimer.stop();
            recentsOverlay.stopOverlay();
            recentsView.x = 0;
            recentsView.y = 0;
            recentsView.enabled = false;
            eventsView.x = width;
            eventsView.y = 0;
            eventsView.visible = false;
            fingerPrint.visible = false;
            stopOverlay();
            openAppFromRecentsViewAnim.stop();
            container.state = "endState";
            openAppFromRecentsViewAnim.start();
            container.endViewIn()
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromRecentsViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: recentsView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: recentsView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                recentsView.visible = false;
            } }
        }
    }
    
    Events {
        id: eventsView
        height: parent.height
        width: parent.width
        x: parent.width
        y: 0
        z: zOrderDesktop
        visible: false
        lockPan: true;
        enabled: false;
    }

    EventsOverlay {
        id: eventsOverlay;

        onEndTutorial: {
            //console.log("eventsView.onEndTutorial");
            eventsView.enabled = false;
            stopOverlay();
            openAppFromEventsViewAnim.stop();
            container.state = "endState";
            openAppFromEventsViewAnim.start();
            container.endViewIn();
        }

        onStartAgain: {
            //console.log("eventsView.onStartAgain");
            eventsView.enabled = false;
            stopOverlay();
            openAppFromEventsViewAnim.stop();
            container.state = "lockState";
            openAppFromEventsViewAnim.start();
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromEventsViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: eventsView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: eventsView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                eventsView.visible = false;
            } }
        }
    }
    
    LockScreen {
        id: endView
        height: parent.height
        width: parent.width
        x: 0
        y: 0
        z: zOrderEnd
        visible: false
        lockSwipe: true;

        onStartHelp: {
            endView.fingerPrintParent = endOverlay;
        }
    }

    EndOverlay {
        id: endOverlay;

        onStartAgain: {
            //console.log("endOverlay.onStartAgain");
            endView.stopView();
            endView.enabled = false;
            stopOverlay();
            openAppFromEndViewAnim.stop();
            container.state = "lockState";
            openAppFromEndViewAnim.start();
            container.endViewOut();
        }

        SequentialAnimation {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            id: openAppFromEndViewAnim;
            ParallelAnimation {
                PropertyAnimation { target: endView; property: "opacity"; to: 0.6; easing.type: Easing.InOutQuint; duration: 600 }
                PropertyAnimation { target: endView; property: "scale"; to: 0.8; easing.type: Easing.InOutQuint; duration: 600 }
            }
            ScriptAction { script: {
                endView.visible = false;
            } }
        }
    }

    states: [
        State { name: "welcomeState"},
        State { name: "lockState"},
        State { name: "launchPhoneState" },
        State { name: "phoneState" },
        State { name: "launcherState" },
        State { name: "recentsOpenPhoneState" },
        State { name: "phoneClosingState" },
        State { name: "recentsState"},
        State { name: "eventsState"   },
        State { name: "endState"}
    ]

    transitions: [
        Transition {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            to: "lockState"
            SequentialAnimation {
                ScriptAction { script: {
                    lockView.x = width;
                    lockView.y = 0;
                    lockView.scale = 1;
                    lockView.opacity = 1;
                    lockView.visible = true;
                    lockOverlay.state = "off";
                } }
                PropertyAnimation { target: lockView; property: "opacity"; to: 1; duration: 300 } // fake anim to wait
                ParallelAnimation {
                    PropertyAnimation { target: lockView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                    PropertyAnimation { target: lockView; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                }
                ScriptAction { script: {
                    lockOverlay.init();
                    lockView.initHelpAnim();
                    lockView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Swipe : Swipe Lockscreen */
            from: "lockState"; to: "launchPhoneState"
            SequentialAnimation {
                ParallelAnimation {
                    PropertyAnimation { target: launchPhoneView; property: "scale"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                    PropertyAnimation { target: launchPhoneView; property: "opacity"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                }
                ScriptAction { script: {
                    launchPhoneOverlay.init();
                    launchPhoneView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Launcher */
            from: "launchPhoneState"; to: "phoneState"
            SequentialAnimation {
                ScriptAction { script: {
                    phoneView.x = width;
                    phoneView.y = 0;
                    phoneView.scale = 1;
                    phoneView.opacity = 1;
                    phoneView.visible = true;
                    phoneOverlay.state = "off";
                } }
                PropertyAnimation { target: phoneView; property: "opacity"; to: 1; duration: 300 } // fake anim to wait
                ParallelAnimation {
                    PropertyAnimation { target: phoneView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                    PropertyAnimation { target: phoneView; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                }
                ScriptAction { script: {
                    phoneOverlay.init();
                    phoneView.initHelpAnim();
                    phoneView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Swipe : iSwipe Normal App */
            from: "phoneState"; to: "launcherState"
            SequentialAnimation {
                ParallelAnimation {
                    PropertyAnimation { target: launcherView; property: "scale"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                    PropertyAnimation { target: launcherView; property: "opacity"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                }
                ScriptAction { script: {
                    launcherOverlay.init();
                    launcherHelpAnimTimer.start();
                    launcherView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Home : Paging */
            from: "launcherState"; to: "recentsOpenPhoneState"
            SequentialAnimation {
                PropertyAnimation { target: recentsOpenPhoneView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: panDuration }
                ScriptAction { script: {
                    recentsOpenPhoneOverlay.init();
                    recentsOpenPhoneView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Launcher */
            from: "recentsOpenPhoneState"; to: "phoneClosingState"
            SequentialAnimation {
                ScriptAction { script: {
                    phoneClosingView.x = width;
                    phoneClosingView.y = 0;
                    phoneClosingView.scale = 1;
                    phoneClosingView.opacity = 1;
                    phoneClosingView.visible = true;
                    phoneClosingOverlay.state = "off";
                } }
                PropertyAnimation { target: phoneClosingView; property: "opacity"; to: 1; duration: 300 } // fake anim to wait
                ParallelAnimation {
                    PropertyAnimation { target: phoneClosingView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                    PropertyAnimation { target: phoneClosingView; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                }
                ScriptAction { script: {
                    phoneClosingOverlay.init();
                    phoneClosingView.initHelpAnim();
                    phoneClosingView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Swipe : iSwipe Normal App */
            from: "phoneClosingState"; to: "recentsState"
            SequentialAnimation {
                ParallelAnimation {
                    PropertyAnimation { target: recentsView; property: "scale"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                    PropertyAnimation { target: recentsView; property: "opacity"; to: 1.0; easing.type: Easing.OutQuint; duration: swipeDuration }
                }
                ScriptAction { script: {
                    recentsOverlay.init();
                    recentsHelpAnimTimer.start();
                    recentsView.enabled = true;
                    recentsView.toggle();
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Home : Paging */
            from: "recentsState"; to: "eventsState"
            SequentialAnimation {
                PropertyAnimation { target: eventsView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: panDuration }
                ScriptAction { script: {
                    eventsOverlay.init();
                    eventsView.enabled = true;
                } }
            }
        },
        Transition {
            /* Motion Guide 2.1 : Application Lifecycle : Open App from Active App */
            to: "endState"
            SequentialAnimation {
                ScriptAction { script: {
                    endView.x = width;
                    endView.y = 0;
                    endView.scale = 1;
                    endView.opacity = 1;
                    endView.visible = true;
                    endOverlay.state = "off";
                } }
                PropertyAnimation { target: endView; property: "opacity"; to: 1; duration: 300 } // fake anim to wait
                ParallelAnimation {
                    PropertyAnimation { target: endView; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                    PropertyAnimation { target: endView; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 300 }
                }
                ScriptAction { script: {
                    endOverlay.init();
                    endView.enabled = true;
                } }
            }
        }
    ]
}
