/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 4; tab-width: 4 -*- */
/* vim:set et ai sw=4 ts=4 sts=4: tw=80 cino="(0,W2s,i2s,t0,l1,:0" */

import QtQuick 1.0
Item {
    id: overlay
    anchors.top: parent.top
    anchors.left: parent.left
    anchors.fill: parent

    state: "hidden"
    visible: false;
    opacity: 0.7;
    z: zOrderOverlay;

    /* https://projects.maemo.org/bugzilla/show_bug.cgi?id=256923#c19 */
    property int overlayWaitTime: 500
    property int overlayFadeInTime: 250
    property int overlayFadeOutTime: 250
    
    property alias shadow: shadow.visible;
    property alias dimmer: shadow.source;
    
    function stopOverlay() {
        overlayTimer.stop();
        state = "hidden";
    }

    Image {
        id: shadow
        width: parent.width;
        height: parent.height;
        source: "../images/meegotouch-tutorial-dimming-half.png";
    }

    states: [
        State { name: "off" },
        State { name: "hidden" },
        State { name: "visible" }
    ]
    transitions:[
        Transition {
            to: "off"
            ScriptAction { script: overlay.visible = false; }
        },
        Transition {
            to: "visible"
            SequentialAnimation {
                ScriptAction { script: {
                    overlay.opacity = 0;
                    overlay.visible = true;
                } }
                PropertyAnimation { target: overlay; property: "opacity"; to: 1.0; easing.type: Easing.OutExpo; duration: overlayFadeInTime }
            }
        },
        Transition {
            to: "hidden"
            SequentialAnimation {
                ScriptAction { script: overlay.opacity = 1; }
                PropertyAnimation { target: overlay; property: "opacity"; to: 0.0; easing.type: Easing.OutExpo; duration: overlayFadeOutTime }
                ScriptAction { script: overlay.visible = false; }
            }
        }
    ]
    
    function init() {
        overlayTimer.start();
    }

    Timer {
        id: overlayTimer
        interval: overlayWaitTime
        onTriggered: overlay.state = "visible";
    }
}

