/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 4; tab-width: 4 -*- */
/* vim:set et ai sw=4 ts=4 sts=4: tw=80 cino="(0,W2s,i2s,t0,l1,:0" */

import QtQuick 1.0
MouseArea {
    id: panable;

    signal panBegin();
    signal panProgress(int dx);
    signal panEnd();
    signal panAbort();

    signal touch();
    signal untouch();

    property int startX: -1;
    property int panToX: 0;
    property bool panning: false;
    property bool lockPan: false;
    
    onPanBegin: {
        panAbortAnim.stop();
    }
    onPanEnd: {
        panAnim.start();
        enabled = false;
    }
    onPanAbort: {
        panAbortAnim.start();
    }

    ParallelAnimation {
        id: panAbortAnim;
        PropertyAnimation { target: panable; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: panDuration }
    }

    SequentialAnimation {
        id: panAnim;
        PropertyAnimation { target: panable; property: "x"; to: panToX; easing.type: Easing.OutQuint; duration: panDuration }
        ScriptAction { script: panable.visible = false; }
    }

    onPressed: {
        touch();
        if (lockPan) return;
        //console.log("PanArea: press on edge x:", mouse.x);
        startX = mouse.x;
    }

    onMousePositionChanged: {
        if (lockPan) return;
        if(0 <= startX && panning == false){
            if ( panThreshold < Math.abs(mouse.x - startX) ) {
                panning = true;
                panBegin();
                //console.log("PanArea: pan begin")
            }
        }
        if ( panning == true ) {
            if ( 0 <= startX ) {
                var dx = mouse.x - startX;
                //console.log("PanArea dx:", dx)
                /* Hard coding: only pan from right to left */
                if(panable.x + dx <= 0) {
                    panable.x += dx;
                    panProgress(dx);
                }
            }
        }
    }

    /*
     * Lets emit untouch() signal in all case except for succesfull gesture.
     */
    onReleased:  {
        if (lockPan) {
            untouch();
            return;
        }
        if(panning == true){
            //console.log("PanArea: released at x:", mouse.x);
            //console.log("PanArea: released at y:", mouse.y);
            if ( -1 < startX && width * cancelShortEdgePan < Math.abs(panable.x) ) {
                //console.log("PanArea: pan on x")
                panToX = width;
                if ( panable.x < 0)
                    panToX = -width;
                panEnd();
            } else {
                //console.log("PanArea: pan aborted")
                untouch();
                panAbort();
            }
        } else {
            untouch();
        }
        panning = false
        startX = -1
    }
}






