/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 4; tab-width: 4 -*- */
/* vim:set et ai sw=4 ts=4 sts=4: tw=80 cino="(0,W2s,i2s,t0,l1,:0" */

import QtQuick 1.0
MouseArea {
    id: swipeable;

    signal swipeBegin();
    signal swipeProgress(real progress);
    signal swipeEnd();
    signal swipeAbort();
    
    /* Used for brutal hack: reparent fingerPrint to Overlay so fingerPrint
     * will be shown above overlay content and not beneath it. */
    signal startHelp();

    signal touch();
    signal untouch();

    property int startX: -1;
    property int startY: -1;
    property int swipeToX: 0;
    property int swipeToY: 0;
    property bool swiping: false;
    property bool lockSwipe: false;
    property int helpMoveLength: 50;

    property alias fingerPrintParent: fingerPrint.parent;
    
    function stopView() {
        helpAnim.stop();
        swipeableTimer.stop();
        x = 0;
        y = 0;
        fingerPrint.visible = false;
    }

    onSwipeBegin: {
        swipeAbortAnim.stop();
        stopView();
    }
    onSwipeEnd: {
        enabled = false;
        swipeAnim.start();
    }
    onSwipeAbort: {
        swipeAbortAnim.start();
        swipeableTimer.start();
    }

    ParallelAnimation {
        id: swipeAbortAnim;
        PropertyAnimation { target: swipeable; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: swipeDuration }
        PropertyAnimation { target: swipeable; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: swipeDuration }
    }
 
    SequentialAnimation {
        id: swipeAnim;
        ParallelAnimation {
            PropertyAnimation { target: swipeable; property: "x"; to: swipeToX; easing.type: Easing.OutQuint; duration: swipeDuration }
            PropertyAnimation { target: swipeable; property: "y"; to: swipeToY; easing.type: Easing.OutQuint; duration: swipeDuration }
        }
        ScriptAction { script: swipeable.visible = false; }
    }

    SequentialAnimation {
        id: helpAnim;

        ScriptAction { script: {
            startHelp();
        } }

        ScriptAction { script: {
            fingerPrint.x = parent.width;
            fingerPrint.y = (parent.height - fingerPrint.height) / 2;
            fingerPrint.opacity = 0.5;
            fingerPrint.visible = true;
        } }
        PropertyAnimation { target: fingerPrint; property: "x"; to: parent.width-fingerPrint.width/2; easing.type: Easing.Linear; duration: 200 }
        ParallelAnimation {
            PropertyAnimation { target: fingerPrint; property: "x"; to: parent.width-fingerPrint.width/2-helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
            PropertyAnimation { target: swipeable; property: "x"; to: -helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
        }
//        PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 150 }
        ScriptAction { script: fingerPrint.visible = false; }
        PropertyAnimation { target: swipeable; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 250 }
        PropertyAnimation { target: swipeable; property: "x"; to: 0; duration: 400 } // hack to wait;

        ScriptAction { script: {
            fingerPrint.x = (parent.width - fingerPrint.width) / 2;
            fingerPrint.y = parent.height;
            fingerPrint.opacity = 0.5;
            fingerPrint.visible = true;
        } }
        PropertyAnimation { target: fingerPrint; property: "y"; to: parent.height-fingerPrint.height/2; easing.type: Easing.Linear; duration: 200 }
        ParallelAnimation {
            PropertyAnimation { target: fingerPrint; property: "y"; to: parent.height-fingerPrint.height/2-helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
            PropertyAnimation { target: swipeable; property: "y"; to: -helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
        }
//        PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 150 }
        ScriptAction { script: fingerPrint.visible = false; }
        PropertyAnimation { target: swipeable; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 250 }
        PropertyAnimation { target: swipeable; property: "y"; to: 0; duration: 400 } // hack to wait;

        ScriptAction { script: {
            fingerPrint.x = - fingerPrint.width;
            fingerPrint.y = (parent.height - fingerPrint.height) / 2;
            fingerPrint.opacity = 0.5;
            fingerPrint.visible = true;
        } }
        PropertyAnimation { target: fingerPrint; property: "x"; to: -fingerPrint.width/2; easing.type: Easing.Linear; duration: 200 }
        ParallelAnimation {
            PropertyAnimation { target: swipeable; property: "x"; to: helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
            PropertyAnimation { target: fingerPrint; property: "x"; to: helpMoveLength-fingerPrint.width/2; easing.type: Easing.OutQuint; duration: 1000 }
        }
//        PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 150 }
        ScriptAction { script: fingerPrint.visible = false; }
        PropertyAnimation { target: swipeable; property: "x"; to: 0; easing.type: Easing.OutQuint; duration: 250 }
        PropertyAnimation { target: swipeable; property: "x"; to: 0; duration: 400 } // hack to wait;

        // swipe from top is closing the app
//        ScriptAction { script: {
//            fingerPrint.x = (parent.width - fingerPrint.width) / 2;
//            fingerPrint.y = - fingerPrint.height;
//            fingerPrint.opacity = 0.5;
//            fingerPrint.visible = true;
//        } }
//        PropertyAnimation { target: fingerPrint; property: "y"; to: -fingerPrint.height/2; easing.type: Easing.Linear; duration: 200 }
//        ParallelAnimation {
//            PropertyAnimation { target: swipeable; property: "y"; to: helpMoveLength; easing.type: Easing.OutQuint; duration: 1000 }
//            PropertyAnimation { target: fingerPrint; property: "y"; to: helpMoveLength-fingerPrint.height/2; easing.type: Easing.OutQuint; duration: 1000 }
//        }
////        PropertyAnimation { target: fingerPrint; property: "opacity"; to: 0; easing.type: Easing.OutExpo; duration: 150 }
//        ScriptAction { script: fingerPrint.visible = false; }
//        PropertyAnimation { target: swipeable; property: "y"; to: 0; easing.type: Easing.OutQuint; duration: 250 }
//        PropertyAnimation { target: swipeable; property: "y"; to: 0; duration: 400 } // hack to wait;

        ScriptAction { script: swipeableTimer.start(); }
    }

    onPressed: {
        touch();
        if (lockSwipe) return;
        if ( mouse.x <= swipeWidth || width - swipeWidth < mouse.x ) {
            //console.log("SwipeArea: press on edge x:", mouse.x);
            startX = mouse.x
        } else if ( mouse.y <= swipeWidth || height - swipeWidth < mouse.y ) {
            //console.log("SwipeArea: press on edge y:", mouse.y);
            startY = mouse.y
        }
    }

    onMousePositionChanged: {
        if (lockSwipe) return;
        if(( (0 <= startX) || (50 <= startY )) && (swiping == false)){
            if ( ( swipeThreshold < Math.abs(mouse.x - startX) ) ||
                 ( swipeThreshold < Math.abs(mouse.y - startY) ) ) {
                swiping = true;
                swipeBegin();
                //console.log("SwipeArea: swipe begin")
            }
        }
        if ( swiping == true ) {
            if ( 0 <= startX ) {
                var dx = mouse.x - startX;
                //console.log("SwipeArea x:", x)
                //console.log("SwipeArea m.x:", mouse.x)
                //console.log("SwipeArea dx:", dx)
                swipeable.x += dx;
                swipeProgress(Math.abs(swipeable.x / width));
            } else if ( 0 <= startY ) {
                var dy = mouse.y - startY;
                //console.log("SwipeArea dy:", dy)
                swipeable.y += dy;
                swipeProgress(Math.abs(swipeable.y / height));
            }
        }
    }

    /*
     * Lets emit untouch() signal in all case except for succesfull gesture.
     */
    onReleased:  {
        if (lockSwipe) {
            untouch();
            return;
        }
        if(swiping == true){
            //console.log("SwipeArea: released at x:", mouse.x);
            //console.log("SwipeArea: released at y:", mouse.y);
            if ( -1 < startX && width * cancelShortEdgeSwipe < Math.abs(swipeable.x) ) {
                //console.log("SwipeArea: swipe on x")
                swipeToY = 0;
                swipeToX = width;
                if ( swipeable.x < 0)
                    swipeToX = -width;
                swipeEnd();
            } else if ( -1 < startY && height * cancelLongEdgeSwipe < Math.abs(swipeable.y) ) {
                //console.log("SwipeArea: swipe on y")
                swipeToX = 0;
                swipeToY = height;
                if ( swipeable.y < 0)
                    swipeToY = -height;
                swipeEnd();
            } else {
                //console.log("SwipeArea: swipe aborted")
                untouch();
                swipeAbort();
            }
        } else {
            untouch();
        }
        swiping = false
        startX = -1
        startY = -1
    }

    function initHelpAnim() {
        swipeableTimer.start();
    }

    Timer {
        id: swipeableTimer;
        interval: 3000;
        onTriggered: helpAnim.start();
    }

    Rectangle {
        id: fingerPrint;
        width: 80;
        height: 80;
        radius: 40;
        color: blueColor;
        visible: false;
        z: zOrderUser;
    }

    Image {
        x: parent.width; y: 0; width: 24; height: parent.height;
        source: "../images/dropshadow.png";
    }

    Image {
        x: 0; y: -parent.height; width: 24; height: parent.height;
        transformOrigin: Item.BottomLeft;
        rotation: -180;
        source: "../images/dropshadow.png";
    }

    Image {
        x: parent.width; y: -parent.width; width: 24; height: parent.width;
        transformOrigin: Item.BottomLeft;
        rotation: -90;
        source: "../images/dropshadow.png";
    }

    Image {
        x: parent.width; y: parent.height; width: 24; height: parent.width;
        transformOrigin: Item.TopLeft;
        rotation: 90;
        source: "../images/dropshadow.png";
    }
}
